require 'spec_helper'

describe 'puppet_enterprise::profile::console::workers' do
  before :each do
    @facter_facts = {
      'osfamily'          => 'Debian',
      'operatingsystem'   => 'Debian',
      'lsbmajdistrelease' => '6',
      'puppetversion'     => '3.6.2 (Puppet Enterprise 3.3.0)',
      'is_pe'             => 'true',
      'fqdn'              => 'console.rspec',
      'clientcert'        => 'awesomecert',
      'pe_concat_basedir'    => '/tmp/file',
    }

    @params = {
      'database_name' => 'console-db',
      'database_user' => 'console-user',
      'database_password' => 'secretpassword',
      'database_host' => 'database.rspec',
      'database_port' => 2345,
    }
  end

  let(:facts) { @facter_facts }
  let(:params) { @params }

  describe "managing the settings file to pass settings validation" do
    let(:file) { '/etc/puppetlabs/puppet-dashboard/settings.yml' }
    it { should contain_file(file).with(:owner => 'puppet-dashboard',
                                        :group => 'puppet-dashboard',
                                        :mode  => '0640') }

    it { should have_yaml_file(file, 'daily_run_history_length' => 30) }
  end

  describe "managing dashboard database config" do
    let(:file) { '/etc/puppetlabs/puppet-dashboard/database.yml' }
    it { should contain_file(file).with(:owner => 'puppet-dashboard',
                                        :group => 'puppet-dashboard',
                                        :mode  => '0640') }

    it { should have_yaml_file(file, 'production' => { 'database'   => 'console-db',
                                                     'username' => 'console-user',
                                                     'password' => 'secretpassword',
                                                     'host'     => 'database.rspec',
                                                     'port'     => 2345,
                                                     'adapter'  => 'postgresql' }) }
  end

  describe "workers should not migrate databases" do
    it { should_not contain_exec('migrate console database').with_command('rake db:migrate').that_requires('File[/etc/puppetlabs/puppet-dashboard/database.yml]') }
  end

  describe "managing dashboard workers defaults" do
    describe "on debian" do
      before(:each) { @facter_facts['osfamily'] = 'Debian' }

      it { should contain_file('/etc/default/pe-puppet-dashboard-workers').with(:owner => 'root',
                                                                                :group => 'root',
                                                                                :mode => '0644') }

      it { should contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/START=yes/) }

      it "defaults to 2 workers" do
        expect(subject).to contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/NUM_DELAYED_JOB_WORKERS=2/)
      end

      it "sets the specified number of workers" do
        @params['delayed_job_workers'] = 5
        expect(subject).to contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/NUM_DELAYED_JOB_WORKERS=5/)
      end
    end

    ['RedHat', 'Suse'].each do |osfamily|
      describe "on #{osfamily}" do
        before(:each) { @facter_facts['osfamily'] = osfamily }

        it { should contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with(:owner => 'root',
                                                                                    :group => 'root',
                                                                                    :mode => '0644') }

        it "defaults to 2 workers" do
          expect(subject).to contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with_content(/CPUS=2/)
        end

        it "sets the specified number of workers" do
          @params['delayed_job_workers'] = 5
          expect(subject).to contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with_content(/CPUS=5/)
        end
      end
    end
  end
end

